@@ -158,10 +158,12 @@ file name and check that the faces of the fonts in the buffer match."
                               (php-mode-test-process-magics))))
      ,(if faces
           `(should (equal
-                    (php-mode-test--parse-list-file
-                     (concat (expand-file-name ,file php-mode-test-dir)
-                             (if (eq t ,faces) ".faces" ,faces)))
-                    (php-mode-test--buffer-face-list (current-buffer)))))
+                    (cons ,file
+                          (php-mode-test--parse-list-file
+                           (concat (expand-file-name ,file php-mode-test-dir)
+                                   (if (eq t ,faces) ".faces" ,faces))))
+                    (cons ,file
+                          (php-mode-test--buffer-face-list (current-buffer))))))
      (goto-char (point-min))
      (let ((case-fold-search nil))
        ,@body)))
@@ -370,25 +372,6 @@ style from Drupal."
 
 (ert-deftest php-mode-test-constants ()
   "Proper highlighting for constants."
-  (custom-set-variables '(php-extra-constants (quote ("extraconstant"))))
-  (with-php-mode-test ("constants.php")
-    (let ((variables '(
-                       "true" "TRUE"
-                       "false" "FALSE"
-                       "null" "NULL"
-                       "IS_CONSTANT"
-                       "__IS_CONSTANT__"
-                       "IS_CONSTANT99"
-                       "extraconstant"
-                       "ClassName")))
-      (dolist (variable variables)
-        (search-forward variable)
-        (goto-char (match-beginning 0))
-        (should (equal (cons variable 'php-constant)
-                       (cons variable (get-text-property (point) 'face)))))))
-
-  ;; Set default
-  (custom-set-variables '(php-extra-constants (quote ())))
   (with-php-mode-test ("constants.php" :faces t)))
 
 (ert-deftest php-mode-test-identifiers()
@@ -440,14 +423,16 @@ style from Drupal."
   (with-php-mode-test ("language-constructs.php")
     (while (search-forward "ClassName" nil t)
       (backward-char)
-      (should (eq 'font-lock-type-face
-                  (get-text-property (point) 'face)))))
+      (let ((token (symbol-at-point)))
+        (should (equal (list token 'font-lock-type-face)
+                       (list token (get-text-property (point) 'face)))))))
   (with-php-mode-test ("language-constructs.php")
     (search-forward "Start:")
     (while (not (= (line-number-at-pos) (count-lines (point-min) (point-max))))
       (forward-line 1)
-      (should (eq 'php-keyword
-                  (get-text-property (point) 'face))))))
+      (let ((token (symbol-at-point)))
+        (should (equal (list token 'php-keyword)
+                       (list token (get-text-property (point) 'face))))))))
 
 (ert-deftest php-mode-test-issue-178 ()
   "Highligth as keyword and following symbol"
@@ -660,6 +645,10 @@ Meant for `php-mode-test-issue-503'."
     (goto-char (point-min))
     (should (eq (php-mode-test-in-function-p nil) nil))))
 
+(ert-deftest php-mode-test-issue-623 ()
+  "Proper alignment object -> accessor."
+  (with-php-mode-test ("indent/issue-623.php" :indent t :magic t)))
+
 (ert-deftest php-mode-test-php74 ()
   "Test highlighting language constructs added in PHP 7.4."
   (with-php-mode-test ("7.4/arrow-function.php" :faces t))
